/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.servlet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Resource;
import org.wso2.registry.Registry;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.registry.secure.RegistryRealm;
import org.wso2.registry.secure.RegistryUMConstants;
import org.wso2.usermanager.Realm;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * Servlet for providing REST API for the registry.
 */
public class RegistryServlet extends HttpServlet {

    private static final Log log = LogFactory.getLog(RegistryServlet.class);
    protected transient ServletConfig servletConfig;

    /**
     * Core registry used in the back-end. Usually this is a JDBCRegistry and there is only one
     * instance of this registry for the whole application. This should be initialized in this
     * servlet and put in to the servlet context so that it can be accessed from any where.
     */

    private EditProcessor editProcessor;

    private ConsoleURLMapper consoleURLMapper;

    //To store the conetxt path
    private String contextRoot = null;

    public void init(ServletConfig config) throws ServletException {

        super.init(config);
        Realm registryRealm;
        Registry coreRegistry = null;
        
        try {

            registryRealm = RegistryRealm.createRegistryRealm();

            String coreRegistryTypeParam = config.getInitParameter(RegistryConstants.REGISTRY_TYPE_PARAMTER);

            if (coreRegistryTypeParam == null ||
                    coreRegistryTypeParam.equals(RegistryConstants.JDBC_REGISTRY_TYPE)) {

                coreRegistry = new JDBCRegistry(registryRealm);

            } else if (coreRegistryTypeParam.equals(RegistryConstants.REMOTE_REGISTRY_TYPE)) {

                // todo: initialize core registry with a remote registry instance.
            }

            this.editProcessor = new EditProcessor();

        } catch (RegistryException e) {
            e.printStackTrace();
            throw new ServletException(e.getMessage());
        }

        config.getServletContext().setAttribute(RegistryConstants.REGISTRY, coreRegistry);
        config.getServletContext().setAttribute(RegistryConstants.REGISTRY_REALM, registryRealm);

        log.info(Messages.getMessage("server.initalized"));
    }

    public void init() throws ServletException {
        if (this.servletConfig != null) {
            init(this.servletConfig);
        }
    }

    protected void doPost(HttpServletRequest httpServletRequest,
                          HttpServletResponse httpServletResponse)
            throws ServletException, IOException {
        initContextRoot(httpServletRequest);

        String uri = httpServletRequest.getRequestURI();
        String p1 = uri.substring(contextRoot.length(), uri.length());

        if (p1.equals("/edit/addResource")) {
            handleFileUpload(httpServletRequest, httpServletResponse);
        } else if (p1.startsWith("/edit")) {
            editProcessor.processEdit(httpServletRequest, httpServletResponse);
        }

        // throw error!
    }

    private void handleFileUpload(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        try {
            Registry registry = Utils.getSecureRegistry(request);

            Resource fileElement =
                    FileUploadUtil.processUpload(request);
            String path = fileElement.getPath();

            registry.put(path, fileElement);
            request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                    "Resource " + path + " is sucessfully added to the registry.");
            response.setContentType("text/html");
            request.getRequestDispatcher("/admin/edit_resource.jsp")
                    .forward(request, response);

        } catch (RegistryException e) {

            e.printStackTrace();
            request.getSession().setAttribute(EditProcessor.STATUS_MESSAGE_NAME,
                    e.getMessage());
            response.setContentType("text/html");
            request.getRequestDispatcher("/admin/edit_resource.jsp")
                    .forward(request, response);
        }
    }

    protected void doGet(HttpServletRequest request,
                         HttpServletResponse response)
            throws ServletException, IOException {

        initContextRoot(request);

        if (consoleURLMapper.mapAndForward(request, response)) {
            return;
        }

        if (processMetadataView(request, response)) {
            return;
        }

        if (processResourceView(request, response)) {
            return;
        }

        // todo: editing UI is not a part of final registry UI. so remove this before the final release.
        editProcessor.processEdit(request, response);
    }

    /**
     * To find the name of the war , so getting that from the request context path
     *
     * @param req :  HttpServletRequest
     */
    public void initContextRoot(HttpServletRequest req) {
        if (contextRoot != null && contextRoot.trim().length() != 0) {
            return;
        }
        String contextPath = req.getContextPath();
        //handling ROOT scenario, for servlets in the default (root) context, this method returns ""
        if (contextPath != null && contextPath.length() == 0) {
            contextPath = "/";
        }
        this.contextRoot = contextPath;
        consoleURLMapper = new ConsoleURLMapper(contextRoot);
    }

    protected void renderView(String jspName,
                              HttpServletRequest httpServletRequest,
                              HttpServletResponse httpServletResponse)
            throws IOException, ServletException {
        httpServletResponse.setContentType("text/html");
        httpServletRequest.getRequestDispatcher(jspName)
                .include(httpServletRequest, httpServletResponse);
    }

    private boolean processResourceView(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String uri = request.getRequestURI();
        String query = request.getQueryString();
        if (query != null && !query.startsWith("v=")) {
            return false;
        }

        if (query == null) {
            query = "";
        } else {
            query = "?" + query;
        }

        // we process only the URL in the form /wso2registry/view/...
        String viewPart = uri.substring(contextRoot.length());
        if (!viewPart.startsWith("/view")) {
            return false;
        }

        if (uri.endsWith("view") || uri.endsWith("view/")) {
            response.setContentType("text/html");
            request.getSession().setAttribute("path", "/" + query);
            request.getRequestDispatcher("/admin/rest.jsp")
                    .include(request, response);
            return true;
        }
        String path = uri.substring((contextRoot + RegistryConstants.TAG_REGISTRY).length(), uri.length());
        path = path + query;

        try {
            Registry registry = Utils.getSecureRegistry(request);
            Resource resource = registry.get(path);

            if (resource != null && !resource.isDirectory()) {
                Object content = resource.getContent();
                if (content != null) {
                    if (content instanceof byte[]) {
                        response.setContentType(resource.getMediaType());
                        response.getOutputStream().write((byte[])content);
                        response.flushBuffer();
                    } else {
                        response.setContentType(resource.getMediaType());
                        response.getWriter().write(content.toString());
                    }
                }

                response.getWriter().flush();
            } else {
                request.getSession().setAttribute("path", path);
                response.setContentType("text/html");
                request.getRequestDispatcher("/admin/rest.jsp")
                        .forward(request, response);
            }
        } catch (RegistryException e) {
            request.getSession().setAttribute("error_message", e.getMessage());
            response.setContentType("text/html");
            request.getRequestDispatcher("/admin/error.jsp")
                    .forward(request, response);
        }

        return true;
    }

    private boolean processMetadataView(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        String query = request.getQueryString();
        if (query != null) {
            if (query.equalsIgnoreCase("tags")) {
                String uri = request.getRequestURI();
                String path = uri.substring(
                        (contextRoot + RegistryConstants.TAG_REGISTRY).length(), uri.length());

                request.getSession().setAttribute(RegistryConstants.PATH, path);
                response.setContentType("text/html");
                request.getRequestDispatcher("/admin/tags.jsp")
                        .forward(request, response);
                return true;
            } else if (query.equalsIgnoreCase("comments")) {
                String uri = request.getRequestURI();
                String path = uri.substring(
                        (contextRoot + RegistryConstants.TAG_REGISTRY).length(), uri.length());

                request.getSession().setAttribute(RegistryConstants.PATH, path);
                response.setContentType("text/html");
                request.getRequestDispatcher("/admin/comments.jsp")
                        .forward(request, response);
                return true;
            }
        }

        return false;
    }
}
